<?php
namespace App\Exports;

use Illuminate\Support\Facades\DB;
use App\Models\leave_apply_emp;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithStyles;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;


class ExportLeaveSummaryReport implements FromCollection, WithHeadings, ShouldAutoSize, WithStyles
{
     protected $data;

    public function __construct(array $data)
    {
        $this->data = $data;
    }

    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {
        // Fetch leave reports based on the provided data
        $all = " AND employee_id =".$this->data['enrol_id'];
        if($this->data['enrol_id'] == 'all') {
        	$all  = "";
        }
        $leaveReports = collect(DB::select("SELECT 
            (SELECT concat(first_name,'',last_name) FROM users WHERE enrol_id = leave_apply_emps.employee_id) as emp_name,
            (SELECT title FROM leave_titles WHERE id =  leave_apply_emps.leave_title_id) as title,
            DATE_FORMAT(start_date, '%Y-%m-%d') as start_date,
            DATE_FORMAT(end_date, '%Y-%m-%d') as end_date,
            total_leave as entitlled,
            SUM(DATEDIFF(end_date, start_date) + 1) as availed,
             total_leave - SUM(DATEDIFF(end_date, start_date) + 1)  as Balanace

             FROM leave_apply_emps WHERE 
             DATE_FORMAT(start_date, '%Y-%m-%d') >= ? 
             AND DATE_FORMAT(end_date, '%Y-%m-%d') <= ? $all  GROUP BY employee_id
              ", [
            $this->data['start_date'],
            $this->data['end_date'],
            
        ]));

        // Map the data to the required format for export
        $formattedData = $leaveReports->map(function ($report) {
            return [
                'Employee Name' => $report->emp_name,
                'Start Date' => $report->start_date,
                'End Date' => $report->end_date,
                'Leave Title' => $report->title,
                'Entitlled' => $report->entitlled,
                'Availed' => $report->availed,
                'Balanace' => $report->Balanace
            ];
        });

        // Add header row
      

        return $formattedData;
    }

    /**
     * @return array
     */
    public function headings(): array
    {
        return [
			'Employee Name',
			'Start Date',
			'End Date',
			'Leave Title',
			'Entitlled',
			'Availed',
			'Balanace'
            // Add more headings as needed
        ];
    }

    /**
     * @param Worksheet $sheet  
     * @return array
     */
    public function styles(Worksheet $sheet)
    {
        return [
            1 => ['font' => ['bold' => true]], // Make the first row bold (heading row)
        ];
    }
}
