<?php

namespace App\Exports;

use Illuminate\Support\Facades\DB;
use Illuminate\Support\Collection;
use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithStyles;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;


class ExportPayrolReport implements FromCollection, WithHeadings, ShouldAutoSize, WithStyles
{

     protected $data;

    public function __construct(array $data)
    {
        $this->data = $data;
    }

    /**
     * @return \Illuminate\Support\Collection
     */
    public function collection()
    {   
        // Fetch leave reports based on the provided data
        $all = " AND employee_id =".$this->data['enrol_id'];
        $yearMonth = $this->data['month'].'-'.$this->data['year'];
        if($this->data['enrol_id'] == 'all') {
        	$all  = "";
        }
        $leaveReports = collect(DB::select("SELECT 
emp_enroll,
emp_name,
(SELECT dept_name as department FROM departments WHERE  id = payroll_data_approval.department ) as department,
salary_package,
gross_salary,   
lates,
late_deduct,
late_deduct_amt,
half_day,
half_day_deduct,
half_day_deduct_amt,
absent,
absent_deduct,
loan,
total_deduction,
bonus_extra_allowance,
fuel_allowance,
net_salary,
sl,
cl,
hd,
yearMonth,
dateTime,
status
 FROM payroll_data_approval WHERE
              yearMonth = ? 
             $all ", [
            $yearMonth,
            
        ]));

        // Map the data to the required format for export
        $formattedData = $leaveReports->map(function ($report) {
            return [
					'emp_enroll'=>$report->emp_enroll,
					'emp_name'=>$report->emp_name,
					'department'=>$report->department,
					'salary_package'=>$report->salary_package,
					'gross_salary'=>$report->gross_salary,
					'lates'=>$report->lates,
					'late_deduct'=>$report->late_deduct,
					'late_deduct_amt'=>$report->late_deduct_amt,
					'half_day'=>$report->half_day,
					'half_day_deduct'=>$report->half_day_deduct,
					'half_day_deduct_amt'=>$report->half_day_deduct_amt,
					'absent'=>$report->absent,
					'absent_deduct'=>$report->absent_deduct,
					'loan'=>$report->loan,
					'total_deduction'=>$report->total_deduction,
					'bonus_extra_allowance'=>$report->bonus_extra_allowance,
					'fuel_allowance'=>$report->fuel_allowance,
					'net_salary'=>$report->net_salary,
					'sl'=>$report->sl,
					'cl'=>$report->cl,
					'hd'=>$report->hd,
					'yearMonth'=>$report->yearMonth,
					'dateTime'=>$report->dateTime,
					'status'=>$report->status
                // Add more fields as needed
            ];
        });

        // Add header row
      

        return $formattedData;
    }

    /**
     * @return array
     */
  public function headings(): array
{
    return [
        'emp_enroll',
        'emp_name',
        'department',
        'salary_package',
        'gross_salary',
        'lates',
        'late_deduct',
        'late_deduct_amt',
        'half_day',
        'half_day_deduct',
        'half_day_deduct_amt',
        'absent',
        'absent_deduct',
        'loan',
        'total_deduction',
        'bonus_extra_allowance',
        'fuel_allowance',
        'net_salary',
        'sl',
        'cl',
        'hd',
        'yearMonth',
        'dateTime',
        'status'
    ];
}

    /**
     * @param Worksheet $sheet  
     * @return array
     */
    public function styles(Worksheet $sheet)
    {
        return [
            1 => ['font' => ['bold' => true]], // Make the first row bold (heading row)
        ];
    }

}
